@@ -2,8 +2,6 @@
 #
 # Copyright © 2014 eNovance
 #
-# Author: Julien Danjou <julien@danjou.info>
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -124,7 +122,14 @@ class IPCLock(locking.Lock):
 
 
 class IPCDriver(coordination.CoordinationDriver):
-    """A IPC based driver."""
+    """A `IPC`_ based driver.
+
+    This driver uses `IPC`_ concepts to provide the coordination driver
+    semantics and required API(s). It **is** missing some functionality but
+    in the future these not implemented API(s) will be filled in.
+
+    .. _IPC: http://en.wikipedia.org/wiki/Inter-process_communication
+    """
 
     _SEGMENT_SIZE = 1024
     _GROUP_LIST_KEY = "GROUP_LIST"
@@ -136,26 +141,20 @@ class IPCDriver(coordination.CoordinationDriver):
         super(IPCDriver, self).__init__()
 
     def _start(self):
-        try:
-            self._group_list = sysv_ipc.SharedMemory(
-                ftok(self._GROUP_LIST_KEY, self._GROUP_PROJECT),
-                sysv_ipc.IPC_CREAT,
-                size=self._SEGMENT_SIZE)
-        except sysv_ipc.ExistentialError:
-            raise
+        self._group_list = sysv_ipc.SharedMemory(
+            ftok(self._GROUP_LIST_KEY, self._GROUP_PROJECT),
+            sysv_ipc.IPC_CREAT,
+            size=self._SEGMENT_SIZE)
         self._lock = self.get_lock(self._INTERNAL_LOCK_NAME)
         self._executor = futures.ThreadPoolExecutor(max_workers=1)
 
     def _stop(self):
         self._executor.shutdown(wait=True)
-
-    def __del__(self):
-        if hasattr(self, "_group_list"):
-            try:
-                self._group_list.detach()
-                self._group_list.remove()
-            except sysv_ipc.ExistentialError:
-                pass
+        try:
+            self._group_list.detach()
+            self._group_list.remove()
+        except sysv_ipc.ExistentialError:
+            pass
 
     def _read_group_list(self):
         data = self._group_list.read(byte_count=2)
@@ -240,7 +239,9 @@ class IPCFutureResult(coordination.CoordAsyncResult):
         try:
             return self._fut.result(timeout=timeout)
         except futures.TimeoutError as e:
-            raise coordination.OperationTimedOut(utils.exception_message(e))
+            coordination.raise_with_cause(coordination.OperationTimedOut,
+                                          utils.exception_message(e),
+                                          cause=e)
 
     def done(self):
         return self._fut.done()
